@@ -12,7 +12,7 @@ from aesara.graph.optdb import OptimizationQuery
 from aesara.tensor.basic_opt import ShapeFeature
 from aesara.tensor.var import TensorVariable
 
-from aeppl.abstract import MeasurableVariable, get_measurable_outputs
+from aeppl.abstract import get_measurable_outputs
 from aeppl.logprob import _logprob
 from aeppl.opt import PreserveRVMappings, logprob_rewrites_db
 from aeppl.utils import rvs_to_value_vars
@@ -114,13 +114,28 @@ def factorized_joint_logprob(
     if extra_rewrites is not None:
         extra_rewrites.optimize(fgraph)
 
-    # This is the updated random-to-value-vars map with the
-    # lifted variables
-    lifted_rv_values = rv_remapper.rv_values
-    replacements = lifted_rv_values.copy()
+    # This is the updated random-to-value-vars map with the lifted/rewritten
+    # variables.  The rewrites are supposed to produce new
+    # `MeasurableVariable`s that are amenable to `_logprob`.
+    updated_rv_values = rv_remapper.rv_values
 
-    # To avoid cloning value variables, we map them to themselves in the
-    # `replacements` `dict` (i.e. entries already existing in `replacments`
+    # Some rewrites also transform the original value variables. This is the
+    # updated map from the new value variables to the original ones, which
+    # we want to use as the keys in the final dictionary output
+    original_values = rv_remapper.original_values
+
+    # When a `_logprob` has been produced for a `MeasurableVariable` node, all
+    # other references to it need to be replaced with its value-variable all
+    # throughout the `_logprob`-produced graphs.  The following `dict`
+    # cumulatively maintains remappings for all the variables/nodes that needed
+    # to be recreated after replacing `MeasurableVariable`s with their
+    # value-variables.  Since these replacements work in topological order, all
+    # the necessary value-variable replacements should be present for each
+    # node.
+    replacements = updated_rv_values.copy()
+
+    # To avoid cloning the value variables, we map them to themselves in the
+    # `replacements` `dict` (i.e. entries already existing in `replacements`
     # aren't cloned)
     replacements.update({v: v for v in rv_values.values()})
 
@@ -133,72 +148,70 @@ def factorized_joint_logprob(
     while q:
         node = q.popleft()
 
-        if not any(o in lifted_rv_values for o in node.outputs):
-            if (
-                isinstance(node.op, MeasurableVariable)
-                and not getattr(node.default_output().tag, "ignore_logprob", False)
-                and warn_missing_rvs
-            ):
-                warnings.warn(
-                    "Found a random variable that was neither among the observations "
-                    f"nor the conditioned variables: {node}"
-                )
+        outputs = get_measurable_outputs(node.op, node)
+
+        if not outputs:
             continue
 
-        if isinstance(node.op, MeasurableVariable):
-
-            outputs = get_measurable_outputs(node.op, node)
-
-            q_rv_value_vars = [
-                replacements[q_rv_var]
-                for q_rv_var in outputs
-                if not getattr(q_rv_var.tag, "ignore_logprob", False)
-            ]
-
-            if not q_rv_value_vars:
-                continue
-
-            # Replace `RandomVariable`s in the inputs with value variables.
-            # Also, store the results in the `replacements` map so that we
-            # don't need to redo these replacements.
-            remapped_vars, _ = rvs_to_value_vars(
-                q_rv_value_vars + list(node.inputs),
-                initial_replacements=replacements,
+        if warn_missing_rvs and any(
+            o not in updated_rv_values
+            for o in outputs
+            if getattr(o.tag, "ignore_logprob", False)
+        ):
+            warnings.warn(
+                "Found a random variable that was neither among the observations "
+                f"nor the conditioned variables: {node}"
             )
-            q_rv_value_vars = remapped_vars[: len(q_rv_value_vars)]
-            value_var_inputs = remapped_vars[len(q_rv_value_vars) :]
+            continue
 
-            q_logprob_var = _logprob(
-                node.op,
-                q_rv_value_vars,
-                *value_var_inputs,
-                **kwargs,
-            )
+        q_rv_value_vars = [
+            replacements[q_rv_var]
+            for q_rv_var in outputs
+            if not getattr(q_rv_var.tag, "ignore_logprob", False)
+        ]
 
-            for q_rv_var in q_rv_value_vars:
-                if q_rv_var.name:
-                    q_logprob_var.name = f"{q_rv_var.name}_logprob"
+        if not q_rv_value_vars:
+            continue
 
-                # Recompute test values for the changes introduced by the
-                # replacements above.
-                if config.compute_test_value != "off":
-                    for node in io_toposort(
-                        graph_inputs((q_logprob_var,)),
-                        (q_logprob_var,),
-                    ):
-                        compute_test_value(node)
+        # Replace `RandomVariable`s in the inputs with value variables.
+        # Also, store the results in the `replacements` map for the nodes
+        # that follow.
+        remapped_vars, _ = rvs_to_value_vars(
+            q_rv_value_vars + list(node.inputs),
+            initial_replacements=replacements,
+        )
+        q_rv_value_vars = remapped_vars[: len(q_rv_value_vars)]
+        value_var_inputs = remapped_vars[len(q_rv_value_vars) :]
 
-                if q_rv_var in logprob_vars:
-                    raise ValueError(
-                        f"More than one logprob factor was assigned to the value var {q_rv_var}"
-                    )
+        q_logprob_vars = _logprob(
+            node.op,
+            q_rv_value_vars,
+            *value_var_inputs,
+            **kwargs,
+        )
 
-                logprob_vars[q_rv_var] = q_logprob_var
+        if not isinstance(q_logprob_vars, (list, tuple)):
+            q_logprob_vars = [q_logprob_vars]
 
-        else:
-            raise NotImplementedError(
-                f"A measure/probability could not be derived for {node}"
-            )
+        for q_rv_var, q_logprob_var in zip(q_rv_value_vars, q_logprob_vars):
+
+            q_rv_var = original_values[q_rv_var]
+
+            if q_rv_var.name:
+                q_logprob_var.name = f"{q_rv_var.name}_logprob"
+
+            if q_rv_var in logprob_vars:
+                raise ValueError(
+                    f"More than one logprob factor was assigned to the value var {q_rv_var}"
+                )
+
+            logprob_vars[q_rv_var] = q_logprob_var
+
+        # Recompute test values for the changes introduced by the
+        # replacements above.
+        if config.compute_test_value != "off":
+            for node in io_toposort(graph_inputs(q_logprob_vars), q_logprob_vars):
+                compute_test_value(node)
 
     return logprob_vars
 
