@@ -1,4 +1,4 @@
-from typing import Dict
+from typing import Dict, Optional
 
 import aesara
 import aesara.tensor as at
@@ -24,6 +24,7 @@ from aesara.tensor.subtensor import (
 )
 from aesara.tensor.var import TensorVariable
 
+from aeppl.abstract import MeasurableVariable
 from aeppl.utils import indices_from_subtensor
 
 inc_subtensor_ops = (IncSubtensor, AdvancedIncSubtensor, AdvancedIncSubtensor1)
@@ -31,10 +32,16 @@ subtensor_ops = (AdvancedSubtensor, AdvancedSubtensor1, Subtensor)
 
 
 class PreserveRVMappings(Feature):
-    r"""Keep track of random variable replacements in a map.
+    r"""Keeps track of random variables and their respective value variables during
+    graph rewrites in `rv_values`
 
-    When a `Variable` that is replaced by optimizations, this `Feature` updates
-    the key entries in a map to reflect the new transformed `Variable`\s.
+    When a random variable is replaced in a rewrite, this `Feature` automatically
+    updates the `rv_values` mapping, so that the new variable is linked to the
+    original value variable.
+
+    In addition this `Feature` provides functionality to manually update a random
+    and/or value variable. A mapping from the transformed value variables to the
+    the original value variables is kept in `original_values`.
     """
 
     def __init__(self, rv_values: Dict[TensorVariable, TensorVariable]):
@@ -47,6 +54,7 @@ class PreserveRVMappings(Feature):
             The ``dict`` is updated in-place.
         """
         self.rv_values = rv_values
+        self.original_values = {v: v for v in rv_values.values()}
 
     def on_attach(self, fgraph):
         if hasattr(fgraph, "preserve_rv_mappings"):
@@ -56,7 +64,42 @@ class PreserveRVMappings(Feature):
 
         fgraph.preserve_rv_mappings = self
 
+    def update_rv_maps(
+        self,
+        old_rv: TensorVariable,
+        new_value: TensorVariable,
+        new_rv: Optional[TensorVariable] = None,
+    ):
+        """Update mappings for a random variable.
+
+        It also creates/updates a map from new value variables to their
+        original value variables.
+
+        Parameters
+        ==========
+        old_rv
+            The random variable whose mappings will be updated.
+        new_value
+            The new value variable that will replace the current one assigned
+            to `old_rv`.
+        new_rv
+            When non-``None``, `old_rv` will also be replaced with `new_rv` in
+            the mappings, as well.
+        """
+        old_value = self.rv_values.pop(old_rv)
+        original_value = self.original_values.pop(old_value)
+
+        if new_rv is None:
+            new_rv = old_rv
+
+        self.rv_values[new_rv] = new_value
+        self.original_values[new_value] = original_value
+
     def on_change_input(self, fgraph, node, i, r, new_r, reason=None):
+        """
+        Whenever a node is replaced during rewrite, we check if it had a value
+        variable associated with it and map it to the new node.
+        """
         r_value_var = self.rv_values.pop(r, None)
         if r_value_var is not None:
             self.rv_values[new_r] = r_value_var
@@ -88,22 +131,29 @@ def incsubtensor_rv_replace(fgraph, node):
     if not isinstance(node.op, inc_subtensor_ops):
         return None  # pragma: no cover
 
-    rv_var = node.inputs[0]
+    rv_var = node.outputs[0]
+    if rv_var not in rv_map_feature.rv_values:
+        return None  # pragma: no cover
 
-    if not (rv_var.owner and isinstance(rv_var.owner.op, RandomVariable)):
+    base_rv_var = node.inputs[0]
+
+    if not (
+        base_rv_var.owner
+        and isinstance(base_rv_var.owner.op, MeasurableVariable)
+        and base_rv_var not in rv_map_feature.rv_values
+    ):
         return None  # pragma: no cover
 
     data = node.inputs[1]
     idx = indices_from_subtensor(getattr(node.op, "idx_list", None), node.inputs[2:])
 
     # Create a new value variable with the indices `idx` set to `data`
-    z = at.set_subtensor(rv_map_feature.rv_values[rv_var][idx], data)
-
-    # Replace `rv_var`'s value variable with the new one
-    rv_map_feature.rv_values[rv_var] = z
+    value_var = rv_map_feature.rv_values[rv_var]
+    new_value_var = at.set_subtensor(value_var[idx], data)
+    rv_map_feature.update_rv_maps(rv_var, new_value_var, base_rv_var)
 
     # Return the `RandomVariable` being indexed
-    return [rv_var]
+    return [base_rv_var]
 
 
 @local_optimizer([BroadcastTo])
@@ -163,10 +213,34 @@ def naive_bcast_rv_lift(fgraph, node):
 
 
 logprob_rewrites_db = SequenceDB()
-logprob_rewrites_db.register("canonicalize", optdb["canonicalize"], -10, "basic")
-rv_sinking_db = EquilibriumDB()
+logprob_rewrites_db.name = "logprob_rewrites_db"
+logprob_rewrites_db.register(
+    "pre-canonicalize", optdb.query("+canonicalize"), -10, "basic"
+)
+
+
+class RVSinkingDB(EquilibriumDB):
+    r"""This `EquilibriumDB` doesn't hide its exceptions.
+
+    By setting `failure_callback` to ``None`` in the `EquilibriumOptimizer`\s
+    that `EquilibriumDB` generates, we're able to directly emit the desired
+    exceptions from within the `LocalOptimization`\s themselves.
+    """
+
+    def query(self, *tags, **kwtags):
+        res = super().query(*tags, **kwtags)
+        res.failure_callback = None
+        return res
+
+
+rv_sinking_db = RVSinkingDB()
+rv_sinking_db.name = "rv_sinking_db"
 rv_sinking_db.register("dimshuffle_lift", local_dimshuffle_rv_lift, -5, "basic")
 rv_sinking_db.register("subtensor_lift", local_subtensor_rv_lift, -5, "basic")
 rv_sinking_db.register("broadcast_to_lift", naive_bcast_rv_lift, -5, "basic")
 rv_sinking_db.register("incsubtensor_lift", incsubtensor_rv_replace, -5, "basic")
+
 logprob_rewrites_db.register("sinking", rv_sinking_db, -10, "basic")
+logprob_rewrites_db.register(
+    "post-canonicalize", optdb.query("+canonicalize"), 10, "basic"
+)
