@@ -72,7 +72,7 @@ class TransformedRV(RandomVariable, metaclass=DistributionMeta):
 
 
 @_logprob.register(TransformedRV)
-def transformed_logprob(op, values, *inputs, name=None, **kwargs):
+def transformed_logprob(op, values, *inputs, use_jacobian=True, **kwargs):
     """Compute the log-likelihood graph for a `TransformedRV`.
 
     We assume that the value variable was back-transformed to be on the natural
@@ -80,16 +80,14 @@ def transformed_logprob(op, values, *inputs, name=None, **kwargs):
     """
     (value,) = values
 
-    logprob = _logprob(op.base_op, values, *inputs, name=name, **kwargs)
+    logprob = _logprob(op.base_op, values, *inputs, **kwargs)
 
-    original_forward_value = op.transform.forward(value, *inputs)
-    jacobian = op.transform.log_jac_det(original_forward_value, *inputs)
+    if use_jacobian:
+        original_forward_value = op.transform.forward(value, *inputs)
+        jacobian = op.transform.log_jac_det(original_forward_value, *inputs)
+        logprob += jacobian
 
-    if name:
-        logprob.name = f"{name}_logprob"
-        jacobian.name = f"{name}_logprob_jac"
-
-    return logprob + jacobian
+    return logprob
 
 
 class DefaultTransformSentinel:
@@ -148,10 +146,7 @@ def transform_values(fgraph: FunctionGraph, node: Node) -> Optional[List[Node]]:
     if value_var.name and getattr(transform, "name", None):
         new_value_var.name = f"{value_var.name}_{transform.name}"
 
-    # Map TransformedRV to new value var and delete old mapping
-    new_rv_var = trans_node.outputs[1]
-    rv_map_feature.rv_values[new_rv_var] = new_value_var
-    del rv_map_feature.rv_values[rv_var]
+    rv_map_feature.update_rv_maps(rv_var, new_value_var, trans_node.outputs[1])
 
     return trans_node.outputs
 
@@ -265,8 +260,8 @@ class LogOddsTransform(RVTransform):
         return at.log(sigmoid_value) + at.log1p(-sigmoid_value)
 
 
-class StickBreaking(RVTransform):
-    name = "stickbreaking"
+class Simplex(RVTransform):
+    name = "simplex"
 
     def forward(self, value, *inputs):
         log_value = at.log(value)
@@ -274,7 +269,7 @@ class StickBreaking(RVTransform):
         return log_value[..., :-1] - shift
 
     def backward(self, value, *inputs):
-        value = at.concatenate([value, -at.sum(value, -1, keepdims=True)])
+        value = at.concatenate([value, -at.sum(value, -1, keepdims=True)], axis=-1)
         exp_value_max = at.exp(value - at.max(value, -1, keepdims=True))
         return exp_value_max / at.sum(exp_value_max, -1, keepdims=True)
 
@@ -408,5 +403,5 @@ TransformedVonMisesRV = create_default_transformed_rv_op(
 )
 TransformedDirichletRV = create_default_transformed_rv_op(
     at.random.dirichlet,
-    StickBreaking(),
+    Simplex(),
 )
